 /*

  This is a part of the LiteStep Shell Source code.

  Copyright (C) 1997-98 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************/
#ifndef __WIN32_LEAN_AND_MEAN__
#define __WIN32_LEAN_AND_MEAN__
#endif
#include <time.h>
#include <stdio.h>
#include <tchar.h>
#include <crtdbg.h>
#include <string.h>
#include <windows.h>
#include "lsapi.h"
#include "../dsapi/dsapi.h"

#define MAX_LINE_LENGTH 4096

char* FindLine(LPCTSTR lpKeyName);
void BangToggleWharf(HWND caller,char* args);
void BangGather(HWND caller,char* args);
void BangVWMDesk(HWND caller,char* args);
void BangVWMUp(HWND caller,char* args);
void BangVWMDown(HWND caller,char* args);
void BangVWMLeft(HWND caller,char* args);
void BangVWMRight(HWND caller,char* args);
void BangVWMNav(HWND caller,char* args);
void BangPopup(HWND caller,char* args);
static BOOL LCReadNextLine (FILE *f, LPSTR szBuffer, DWORD dwLength);

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes){
	 return TRUE;
}

// Wrapper functions for DSAPI
int GetRCInt(LPCTSTR lpKeyName, int nDefault){
	int val;
	
	val = RCReadInt(GetRCPath(), _T("LiteStep"), lpKeyName, nDefault);
	return val;
}

BOOL GetRCBool(LPCTSTR lpKeyName, BOOL ifFound){

	if (RCReadBool(GetRCPath(), _T("LiteStep"), lpKeyName))
		return(ifFound);
	else
		return(!ifFound);
}

BOOL GetRCString(LPCTSTR lpKeyName, LPSTR value, LPTSTR defStr, int maxLen){
	LPTSTR val;	

	val = RCReadString(GetRCPath(), _T("LiteStep"), lpKeyName, defStr);
	_tcscpy(value, val);
	if (!_tcscmp(val, defStr))
		return(FALSE);
	else 
		return TRUE;
}

COLORREF GetRCColor(LPCTSTR lpKeyName, COLORREF colDef){
	COLORREF val;
	
	val = RCReadColor(GetRCPath(), _T("LiteStep"), lpKeyName, colDef);
	return(val);
}

HWND GetLitestepWnd(){
	return(GetShellWindow());
}

BOOL AddBangCommand(char *command, void *f) {
	return(AddBang(command, f));
}

BOOL AddBangCommandEx(char *command, void * f) {
	return(AddBang(command, f));
}

BOOL RemoveBangCommand(char *command){
	return(DelBang(command));
}

BOOL ParseBangCommand(HWND caller, char *command, char *args){
	return(BangExec(caller, command, args));
}

HBITMAP LoadLSImage(LPTSTR szImage, LPTSTR szFile){
	HBITMAP hbmp = GetImage(szImage);
	return (hbmp);
}

void VarExpansion(char *buffer, const char * value){
	LPTSTR final;

	final = ExpandVars(value);
	_tcscpy(buffer, final);
}

void GetLSBitmapSize(HBITMAP hBitmap, int *x, int *y){
	GetImageSize(hBitmap, x, y);
}

void TransparentBltLS(HDC dc, int nXDest, int nYDest, int nWidth, 
	int nHeight, HDC tempDC, int nXSrc, int nYSrc,
	COLORREF colorTransparent) {
	TransDraw(dc, nXDest, nYDest, nWidth, nHeight, tempDC, nXSrc, nYSrc, colorTransparent);
}

HBITMAP BitmapFromIcon(HICON hIcon){
	return (IcoToBmp(hIcon));
}

HRGN BitmapToRegion(HBITMAP hBmp, COLORREF xColor, COLORREF cTolerance, int xoffset, int yoffset){
	return (BmpToRgn(hBmp, xColor, cTolerance, xoffset, yoffset));
}

HICON LoadLSIcon(LPCSTR szImage, LPCSTR szFile){
	return (GetIcon(szImage));
}

FILE *LCOpen(LPCTSTR szPath){
	FILE *f = _tfopen(GetRCPath(), _T("r"));
	return f;
}

BOOL LCClose(FILE *f){
	return TRUE;
}

BOOL LCReadNextConfig(FILE *f, LPCTSTR szPrefix, LPTSTR szBuffer, DWORD dwLength){
	LPTSTR buf, final;
	static LONG pos = 0;

	fclose(f);
	buf = RCReadValue(GetRCPath(), _T("LiteStep"), szPrefix, &pos);
	final = ExpandVars(buf);
	if (final) {
		_stprintf(szBuffer, _T("%s %s"), szPrefix, final); 
		return TRUE;
	} else {
		pos = 0;
		return FALSE;
	}
}

BOOL WINAPI LSGetImagePath(LPTSTR pszPath, UINT nMaxLen){
	static LPTSTR szPath = NULL;

	if(!szPath){
		UINT nLen;

		szPath = RCReadString(GetRCPath(), _T("Config"), _T("LSImageFolder"), _T(""));
		if(_tcslen(szPath)==0){
			// backwards compatibility (will be removed eventually)
			szPath = RCReadString(GetRCPath(), _T("Config"), _T("PixmapPath"), _T(""));
			if(_tcslen(szPath)==0){
				szPath = GetShellPath();
				if (szPath[0])
					_tcscat(szPath, TEXT("IMAGES\\"));
			}
		}
		nLen = _tcslen(szPath);
		if (nLen && szPath[nLen - 1] != '\\')
			_tcscat(szPath, TEXT("\\"));
	}
	_tcsncpy(pszPath, szPath, nMaxLen);
	return TRUE;
}
// Stuff not yet wrapped (FAF)
BOOL GetRCLine(LPCTSTR pszCommand, LPTSTR pszBuffer, UINT nBufLen, LPCTSTR pszDefault){
	LPTSTR pszLine = NULL;

	// verify parameters
	if( !pszBuffer || !nBufLen )
		return FALSE;

	// find it
	if( pszCommand )
		pszLine = FindLine(pszCommand);

	if( !pszLine )
	{
		if( pszDefault )
			strncpy( pszBuffer, pszDefault, nBufLen );
		else
			*pszBuffer = 0;

		return FALSE;
	}

	// read past command
	while( *pszLine && *pszLine != ' ' && *pszLine != '\t' )
		pszLine++;

	// read past whitespace
	while( *pszLine && (*pszLine == ' ' || *pszLine == '\t') )
		pszLine++;

	nBufLen--;

	// read in data
	while( *pszLine )
	{
		if( nBufLen-- )
			*pszBuffer++ = *pszLine;

		pszLine++;
	}

	*pszBuffer = 0;
	return TRUE;
}

BOOL LCReadNextCommand (FILE *f, LPSTR szBuffer, DWORD dwLength)
{
	char szTempBuffer[MAX_LINE_LENGTH];

	if (!f)
	{
		return FALSE;
	}

	while (LCReadNextLine (f, szTempBuffer, sizeof (szTempBuffer)))
	{
		if (!ispunct (szTempBuffer[0]))
		{
			strncpy (szBuffer, szTempBuffer, dwLength);

			return TRUE;
		}
	}

	return FALSE;
}

BOOL LCReadNextLine (FILE *f, LPSTR szBuffer, DWORD dwLength)
{
	char szTempBuffer[MAX_LINE_LENGTH];

	while (f && !feof (f))
	{
		int length;

		strcpy(szBuffer, "");

		if (!fgets (szTempBuffer, dwLength, f))
		{
			break;
		}

		length = strlen (szTempBuffer);

		// Skip any leading and trailing whitespace
		while (length && isspace (szTempBuffer[0]))
		{
			length--;
			strcpy (szTempBuffer, szTempBuffer + 1);
		}

		while (length && isspace (szTempBuffer[length-1]))
		{
			szTempBuffer[--length] = '\0';
		}

		if (length && szTempBuffer[0] != ';')
		{
//			if (RCOpened){
//				VarExpansion(szBuffer, szTempBuffer);
//			} else {
				strcpy(szBuffer, szTempBuffer);
//			}
			return TRUE;
		}
	}

	return FALSE;
}

char* FindLine(LPCTSTR lpKeyName){
	TCHAR szCommand[64];
	LPTSTR pszCommand;
	LPTSTR pszBuffer;
	int i;
	int nLen;

/*	if (ThemeInUse){
		for (i = 0; i < ThemeLen; i++)
		{
			pszBuffer = ThemeBuffer[i];
			pszCommand = szCommand;
			nLen = 63;

			while( *pszBuffer && *pszBuffer != ' ' && *pszBuffer != '\t' )
			{
				if( nLen )
				{
					*pszCommand++ = *pszBuffer;
					nLen--;
				}

				pszBuffer++;
			}

			*pszCommand = 0;

			if( !stricmp( szCommand, lpKeyName ) )
				return ThemeBuffer[i];
		}
	}*/

/*	for (i = 0; i < RCLen; i++){
		pszBuffer = RCBuffer[i];
		pszCommand = szCommand;
		nLen = 63;

		while( *pszBuffer && *pszBuffer != ' ' && *pszBuffer != '\t' )
		{
			if( nLen )
			{
				*pszCommand++ = *pszBuffer;
				nLen--;
			}

			pszBuffer++;
		}

		*pszCommand = 0;

		if( !stricmp( szCommand, lpKeyName ) )
			return RCBuffer[i];
	}*/

	return NULL;
}

// These need to be put in Bangs (FAF)
void BangGather(HWND caller,char* args){
	HWND ls = GetLitestepWnd();

	if (ls){
		SendMessage(ls, LM_BRINGTOFRONT, 1, 0);
	}
	return;
}

void BangToggleWharf(HWND caller,char* args){
    HWND ls = GetLitestepWnd();

    if (ls){
		SendMessage(ls, LM_SHADETOGGLE, 0, 0);
	}
    return;
}

void BangVWMDesk(HWND caller,char* args){
	HWND ls = GetLitestepWnd();
	if (ls && args != NULL){
		SendMessage(ls,LM_SWITCHTON,atoi(args),0);
	}
	return;
}

void BangVWMUp(HWND caller,char* args){
	HWND ls = GetLitestepWnd();

	if (ls){
		SendMessage(ls,LM_VWMUP,0,0);
	}
	return;
}

void BangVWMDown(HWND caller,char* args){
	HWND ls = GetLitestepWnd();

	if (ls){
		SendMessage(ls,LM_VWMDOWN,1,0);
	}
	return;
}

void BangVWMLeft(HWND caller,char* args){
	HWND ls = GetLitestepWnd();

	if (ls){
		SendMessage(ls,LM_VWMLEFT,1,0);
	}
	return;
}

void BangVWMRight(HWND caller,char* args){
	HWND ls = GetLitestepWnd();

	if (ls){
		SendMessage(ls,LM_VWMRIGHT,1,0);
	}
	return;
}

void BangVWMNav(HWND caller,char* args){
	HWND ls = GetLitestepWnd();

	if (ls){
		SendMessage(ls,LM_VWMNAV,1,0);
	}
	return;
}

// No need to wrap these
int LCTokenize (LPCSTR szString, LPSTR *lpszBuffers, DWORD dwNumBuffers, LPSTR szExtraParameters)
{
	int		index = 0;
	char	quoteChar = 0;

	char	buffer[MAX_LINE_LENGTH];
	char	output[MAX_LINE_LENGTH];
	char	*pOutput = NULL;
	DWORD	dwBufferCount = 0;

	strcpy (buffer, szString);

	pOutput = output;

	while (buffer[index] && dwBufferCount < dwNumBuffers)
	{
		BOOL skipWhitespace = FALSE;

		switch (buffer[index])
		{
		case '"':
		case '\'':
			{
				if (!quoteChar)
				{
					quoteChar = buffer[index];
					break;
				}
				else
				{
					if (quoteChar == buffer[index])
					{
						quoteChar = 0;
						strcpy (*lpszBuffers, output);
						lpszBuffers++;
						dwBufferCount++;

						if (dwBufferCount < dwNumBuffers)
						{
							(*lpszBuffers)[0] = '\0';
						}
						pOutput = output;
						*pOutput = '\0';
						skipWhitespace = TRUE;
						break;
					}
					else
					{
						*pOutput++ = buffer[index];
						*pOutput = '\0';
						break;
					}
				}
			}
		case ' ':
		case '\t':
			{
				if (!quoteChar)
				{
					if (strlen (output))
					{
						strcpy (*lpszBuffers, output);
						lpszBuffers++;
						dwBufferCount++;
						if (dwBufferCount < dwNumBuffers)
						{
							(*lpszBuffers)[0] = '\0';
						}
						pOutput = output;
						*pOutput = '\0';
						skipWhitespace = TRUE;
					}
				}
				else
				{
					*pOutput++ = buffer[index];
					*pOutput = '\0';
				}
				break;
			}
		default:
			{
				*pOutput++ = buffer[index];
				*pOutput = '\0';
				break;
			}
		}

		index++;

		if (skipWhitespace)
		{
			while (isspace (buffer[index]))
			{
				index++;
			}
		}
	}

	if (strlen (output))
	{
		if (dwBufferCount < dwNumBuffers)
		{
			dwBufferCount++;

			strcat (*lpszBuffers, output);
		}
	}

	if (szExtraParameters && dwBufferCount == dwNumBuffers)
	{
		strcpy (szExtraParameters, buffer + index);
	}

	return dwBufferCount;
}

HINSTANCE LSExecuteEx(HWND Owner, LPSTR szOperation, LPSTR szCommand, LPSTR szArgs, LPSTR szDirectory, int nShowCmd)
{
  if(*szCommand == '!')
    return ParseBangCommand(Owner, szCommand, szArgs) ? (HINSTANCE)1 : (HINSTANCE)0;
  else
    return (HINSTANCE)ShellExecute(Owner, szOperation, szCommand, szArgs, szDirectory, nShowCmd ? nShowCmd : SW_SHOWNORMAL);
}

HINSTANCE LSExecute(HWND Owner, LPCSTR szCommand, int nShowCmd)
{
	char command[4096], *newcmd, *args, *p, dir[_MAX_DIR], full_directory[_MAX_DIR + _MAX_DRIVE + 1]; // Can't rely on MAX_PATH for command
  HINSTANCE val = 0;

	if(!szCommand || strlen(szCommand) < 1)
		return 0;

  VarExpansion(command, szCommand);

  if(*command == '\"') {
    newcmd = command + sizeof(char);
    newcmd = strtok(newcmd, "\"");
    args = strtok(NULL, "\0");
    
    if(!newcmd)
      return 0;
  } else {
    newcmd = strtok(command, "\t ");
    if(!newcmd) {
      newcmd = command;
      args = NULL;
    } else {
      args = strtok(NULL, "\0");
    }
  }

  p = newcmd + (strlen(newcmd) - 1)*sizeof(char);
  while(isspace(*newcmd))
    ++newcmd;
  while(isspace(*p)) {
    *p = '\0';
    --p;
  }

  if(*newcmd != '!') {
    if(args) {
      p = args + (strlen(args) - 1)*sizeof(char);
      while(isspace(*args) || *args == '\"')
        ++args;
      while(isspace(*p) || *p == '\"') {
        *p = '\0';
        --p;
      }
    }
  } else
    if(args) {
      p = args + (strlen(args) - 1)*sizeof(char);
      while(isspace(*args))
        ++args;
      while(isspace(*p)) {
        *p = '\0';
        --p;
      }
    }
	
  if(*newcmd == '!') {
    val = LSExecuteEx(Owner, NULL, newcmd, args, NULL, 0);
  } else {
    if(GetFileAttributes(newcmd) & FILE_ATTRIBUTE_DIRECTORY)
      val = LSExecuteEx(Owner, "open", newcmd, args, NULL, nShowCmd ? nShowCmd : SW_SHOWNORMAL);
    else {
      _splitpath(newcmd, full_directory, dir, NULL, NULL);
      strcat(full_directory, dir);
      val = LSExecuteEx(Owner, "open", newcmd, args, full_directory, nShowCmd ? nShowCmd : SW_SHOWNORMAL);
    }
  }

  free(command);
  return val;
}

void SetDesktopArea(int left, int top, int right, int bottom) {
	RECT r;

	r.left = left;
	r.top = top;
	r.right = right;
	r.bottom = bottom;
	SystemParametersInfo(SPI_SETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
	SystemParametersInfo(SPI_GETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
}

void Frame3D(HDC dc, RECT rect, COLORREF TopColor, COLORREF BottomColor, int Width)
{
	HPEN hPen1 = CreatePen(PS_SOLID, 1, TopColor), 
		hPen2 = CreatePen(PS_SOLID, 1, BottomColor),
		OldPen;
	POINT points[3];

	rect.bottom--;
	rect.right--;

	OldPen = (HPEN)SelectObject(dc, hPen1);

	while (Width > 0)
	{
		POINT p;
		Width--;

		points[0].x = rect.left;
		points[0].y = rect.bottom;
		points[1].x = rect.left;
		points[1].y = rect.top;
		points[2].x = rect.right;
		points[2].y = rect.top;

		Polyline(dc, points, 3);

		SelectObject(dc, hPen2);

		p = points[0];
		points[0] = points[2];
		points[1].x = rect.bottom;
		points[1].y = rect.right;
		points[2] = p;
		points[2].x--;

		Polyline(dc, points, 3);

		SelectObject(dc, hPen1);

		InflateRect(&rect, -1, -1);
	}
	SelectObject(dc, OldPen);
	DeleteObject(hPen1);
	DeleteObject(hPen2);
}